# -----------------------------------------------------------------------------
# (C) British Crown Copyright 2017-2021 Met Office.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 

# Various functions to manupulate SIMIP budget data

# extract_years
#----------------
# Extract a subset of years from ice or snow budget data

# annual_mn_evolution
# -------------------
# Generates the annual mean evolution for the budget terms, ice area and mass
# Returns the ensemble mean and st dev in each case (st dev zero if just one ensemble member)

import numpy as np
import read_budgetfiles as rb

#---------------------------------------------------------------------
#---------------------------------------------------------------------
#---------------------------------------------------------------------
def extract_years(years,area,mass,massbudget,startyear,endyear):
# Extract data for a range of years (assumed already read in)
# Works for ice or snow

    years_index=np.where(np.logical_and (years>=startyear, years<=endyear))
    years_subset = years[years_index] 
    area_subset=np.squeeze(area[years_index,:])
    mass_subset=np.squeeze(mass[years_index,:])
    massbudget_subset=np.squeeze(massbudget[years_index,:,:])
    
    return years_subset,area_subset,mass_subset,massbudget_subset


#---------------------------------------------------------------------
#---------------------------------------------------------------------
#---------------------------------------------------------------------
def annual_mn_evolution(filepath,histfiles,nhistfiles,scenariofiles,nscenariofiles,nskip):

	# -----------------------------------------------------------------------------
	# Loop over the Hist files then loop over the scenario files
	# to make the annual mn and st dev for each budget component, plus the ice area and mass

	seconds_in_year = 60*60*24*30*12

	# Hist files --------------------------
	# Find the no of years
	years,months,icearea,icemass,massbudget = rb.read_icefile(filepath+histfiles[0],nskip)
	nyears=len(years)
	massbudgets_annualmn = np.empty([nhistfiles,nyears,9],dtype='float')
	icearea_annualmn = np.empty([nhistfiles,nyears],dtype='float')
	icemass_annualmn = np.empty([nhistfiles,nyears],dtype='float')


	# Make the annual mean budget for each ensemble member
	for n in range(0,nhistfiles):
		years_Hist,months,icearea,icemass,massbudget = rb.read_icefile(filepath+histfiles[n],nskip)
		massbudgets_annualmn[n,:,:] = seconds_in_year*np.mean(massbudget,axis=1) # Mean over months
		icearea_annualmn[n,:] = np.mean(icearea,axis=1) # Mean over months
		icemass_annualmn[n,:] = np.mean(icemass,axis=1) # Mean over months

	# Make the ensemble mean and st dev 
	massbudget_Hist_mn=np.mean(massbudgets_annualmn,axis=0)
	massbudget_Hist_stdev=np.std(massbudgets_annualmn,axis=0)
	icearea_Hist_mn=np.mean(icearea_annualmn,axis=0)
	icearea_Hist_stdev=np.std(icearea_annualmn,axis=0)	
	icemass_Hist_mn=np.mean(icemass_annualmn,axis=0)
	icemass_Hist_stdev=np.std(icemass_annualmn,axis=0)	


	# Include the Scenario files (if there are any....)----------------------------
	# Find the no of years
	if nscenariofiles > 0:
		years,months,icearea,icemass,massbudget = rb.read_icefile(filepath+scenariofiles[0],nskip)
		nyears=len(years)
		massbudgets_annualmn = np.empty([nscenariofiles,nyears,9],dtype='float')
		icearea_annualmn = np.empty([nscenariofiles,nyears],dtype='float')
		icemass_annualmn = np.empty([nscenariofiles,nyears],dtype='float')

		# Make the annual mean budget for each ensemble member
		for n in range(0,nscenariofiles):
			years_Scenario,months,icearea,icemass,massbudget = rb.read_icefile(filepath+scenariofiles[n],nskip)
			massbudgets_annualmn[n,:,:] = seconds_in_year*np.mean(massbudget,axis=1) # Mean over months
			icearea_annualmn[n,:] = np.mean(icearea,axis=1) # Mean over months
			icemass_annualmn[n,:] = np.mean(icemass,axis=1) # Mean over months

		# Make the ensemble mean and st dev
		massbudget_Scenario_mn=np.mean(massbudgets_annualmn,axis=0)
		massbudget_Scenario_stdev=np.std(massbudgets_annualmn,axis=0)
		icearea_Scenario_mn=np.mean(icearea_annualmn,axis=0)
		icearea_Scenario_stdev=np.std(icearea_annualmn,axis=0)	
		icemass_Scenario_mn=np.mean(icemass_annualmn,axis=0)
		icemass_Scenario_stdev=np.std(icemass_annualmn,axis=0)	

		# Combine the Hist and Scenario segments
		years=np.append(years_Hist,years_Scenario,axis=0)
		massbudget_mn=np.append(massbudget_Hist_mn,massbudget_Scenario_mn,axis=0)
		massbudget_stdev=np.append(massbudget_Hist_stdev,massbudget_Scenario_stdev,axis=0)
		icearea_mn=np.append(icearea_Hist_mn,icearea_Scenario_mn,axis=0)
		icearea_stdev=np.append(icearea_Hist_stdev,icearea_Scenario_stdev,axis=0)
		icemass_mn=np.append(icemass_Hist_mn,icemass_Scenario_mn,axis=0)
		icemass_stdev=np.append(icemass_Hist_stdev,icemass_Scenario_stdev,axis=0)

	# Otherwise just use the Hist section
	else:
		years = years_Hist
		massbudget_mn = massbudget_Hist_mn
		massbudget_stdev = massbudget_Hist_stdev
		icearea_mn = icearea_Hist_mn
		icearea_stdev = icearea_Hist_stdev
		icemass_mn = icemass_Hist_mn
		icemass_stdev = icemass_Hist_stdev


	return years,icearea_mn,icearea_stdev,icemass_mn,icemass_stdev,massbudget_mn,massbudget_stdev
#---------------------------------------------------------------------
#---------------------------------------------------------------------
#---------------------------------------------------------------------
def annual_mn_snow_evolution(filepath,histfiles,nhistfiles,scenariofiles,nscenariofiles,nskip):

	# -----------------------------------------------------------------------------
	# Loop over the Hist files then loop over the scenario files
	# to make the annual mn and st dev for each budget component, plus the snow area and mass

	seconds_in_year = 60*60*24*30*12

	# Hist files --------------------------
	# Find the no of years
	years,months,snowarea,snowmass,massbudget = rb.read_snowfile(filepath+histfiles[0],nskip)
	nyears=len(years)
	massbudgets_annualmn = np.empty([nhistfiles,nyears,7],dtype='float')
	snowarea_annualmn = np.empty([nhistfiles,nyears],dtype='float')
	snowmass_annualmn = np.empty([nhistfiles,nyears],dtype='float')


	# Make the annual mean budget for each ensemble member
	for n in range(0,nhistfiles):
		years_Hist,months,snowarea,snowmass,massbudget = rb.read_snowfile(filepath+histfiles[n],nskip)
		massbudgets_annualmn[n,:,:] = seconds_in_year*np.mean(massbudget,axis=1) # Mean over months
		snowarea_annualmn[n,:] = np.mean(snowarea,axis=1) # Mean over months
		snowmass_annualmn[n,:] = np.mean(snowmass,axis=1) # Mean over months

	# Make the ensemble mean and st dev 
	massbudget_Hist_mn=np.mean(massbudgets_annualmn,axis=0)
	massbudget_Hist_stdev=np.std(massbudgets_annualmn,axis=0)
	snowarea_Hist_mn=np.mean(snowarea_annualmn,axis=0)
	snowarea_Hist_stdev=np.std(snowarea_annualmn,axis=0)	
	snowmass_Hist_mn=np.mean(snowmass_annualmn,axis=0)
	snowmass_Hist_stdev=np.std(snowmass_annualmn,axis=0)	


	# Include the Scenario files (if there are any....)----------------------------
	# Find the no of years
	if nscenariofiles > 0:
		years,months,snowarea,snowmass,massbudget = rb.read_snowfile(filepath+scenariofiles[0],nskip)
		nyears=len(years)
		massbudgets_annualmn = np.empty([nscenariofiles,nyears,7],dtype='float')
		snowarea_annualmn = np.empty([nscenariofiles,nyears],dtype='float')
		snowmass_annualmn = np.empty([nscenariofiles,nyears],dtype='float')

		# Make the annual mean budget for each ensemble member
		for n in range(0,nscenariofiles):
			years_Scenario,months,snowarea,snowmass,massbudget = rb.read_snowfile(filepath+scenariofiles[n],nskip)
			massbudgets_annualmn[n,:,:] = seconds_in_year*np.mean(massbudget,axis=1) # Mean over months
			snowarea_annualmn[n,:] = np.mean(snowarea,axis=1) # Mean over months
			snowmass_annualmn[n,:] = np.mean(snowmass,axis=1) # Mean over months

		# Make the ensemble mean and st dev
		massbudget_Scenario_mn=np.mean(massbudgets_annualmn,axis=0)
		massbudget_Scenario_stdev=np.std(massbudgets_annualmn,axis=0)
		snowarea_Scenario_mn=np.mean(snowarea_annualmn,axis=0)
		snowarea_Scenario_stdev=np.std(snowarea_annualmn,axis=0)	
		snowmass_Scenario_mn=np.mean(snowmass_annualmn,axis=0)
		snowmass_Scenario_stdev=np.std(snowmass_annualmn,axis=0)	

		# Combine the Hist and Scenario segments
		years=np.append(years_Hist,years_Scenario,axis=0)
		massbudget_mn=np.append(massbudget_Hist_mn,massbudget_Scenario_mn,axis=0)
		massbudget_stdev=np.append(massbudget_Hist_stdev,massbudget_Scenario_stdev,axis=0)
		snowarea_mn=np.append(snowarea_Hist_mn,snowarea_Scenario_mn,axis=0)
		snowarea_stdev=np.append(snowarea_Hist_stdev,snowarea_Scenario_stdev,axis=0)
		snowmass_mn=np.append(snowmass_Hist_mn,snowmass_Scenario_mn,axis=0)
		snowmass_stdev=np.append(snowmass_Hist_stdev,snowmass_Scenario_stdev,axis=0)

	# Otherwise just use the Hist section
	else:
		years = years_Hist
		massbudget_mn = massbudget_Hist_mn
		massbudget_stdev = massbudget_Hist_stdev
		snowarea_mn = snowarea_Hist_mn
		snowarea_stdev = snowarea_Hist_stdev
		snowmass_mn = snowmass_Hist_mn
		snowmass_stdev = snowmass_Hist_stdev


	return years,snowarea_mn,snowarea_stdev,snowmass_mn,snowmass_stdev,massbudget_mn,massbudget_stdev

#---------------------------------------------------------------------
#---------------------------------------------------------------------
#---------------------------------------------------------------------
def monthly_mn_evolution(filepath,histfiles,nhistfiles,scenariofiles,nscenariofiles,nskip,month):

	# -----------------------------------------------------------------------------
	# Loop over the Hist files then loop over the scenario files
	# to make the monthly mn and st dev for a specified month,
	# for each budget component, plus the ice area and mass

	# Hist files --------------------------
	# Find the no of years
	years,months,icearea,icemass,massbudget = rb.read_icefile(filepath+histfiles[0],nskip)
	nyears=len(years)
	massbudgets_month = np.empty([nhistfiles,nyears,9],dtype='float')
	icearea_month = np.empty([nhistfiles,nyears],dtype='float')
	icemass_month = np.empty([nhistfiles,nyears],dtype='float')


	# Make the annual mean budget for each ensemble member
	for n in range(0,nhistfiles):
		years_Hist,months,icearea,icemass,massbudget = rb.read_icefile(filepath+histfiles[n],nskip)
		massbudgets_month[n,:,:] = massbudget[:,month-1,:]
		icearea_month[n,:] = icearea[:,month-1]
		icemass_month[n,:] = icemass[:,month-1]

	# Make the ensemble mean and st dev 
	massbudget_Hist_mn=np.mean(massbudgets_month,axis=0)
	massbudget_Hist_stdev=np.std(massbudgets_month,axis=0)
	icearea_Hist_mn=np.mean(icearea_month,axis=0)
	icearea_Hist_stdev=np.std(icearea_month,axis=0)	
	icemass_Hist_mn=np.mean(icemass_month,axis=0)
	icemass_Hist_stdev=np.std(icemass_month,axis=0)	


	# Include the Scenario files (if there are any....)----------------------------
	# Find the no of years
	if nscenariofiles > 0:
		years,months,icearea,icemass,massbudget = rb.read_icefile(filepath+scenariofiles[0],nskip)
		nyears=len(years)
		massbudgets_month = np.empty([nscenariofiles,nyears,9],dtype='float')
		icearea_month = np.empty([nscenariofiles,nyears],dtype='float')
		icemass_month = np.empty([nscenariofiles,nyears],dtype='float')

		# Make the annual mean budget for each ensemble member
		for n in range(0,nscenariofiles):
			years_Scenario,months,icearea,icemass,massbudget = rb.read_icefile(filepath+scenariofiles[n],nskip)
			massbudgets_month[n,:,:] = massbudget[:,month-1,:]
			icearea_month[n,:] = icearea[:,month-1]
			icemass_month[n,:] = icemass[:,month-1]


		# Make the ensemble mean and st dev
		massbudget_Scenario_mn=np.mean(massbudgets_month,axis=0)
		massbudget_Scenario_stdev=np.std(massbudgets_month,axis=0)
		icearea_Scenario_mn=np.mean(icearea_month,axis=0)
		icearea_Scenario_stdev=np.std(icearea_month,axis=0)	
		icemass_Scenario_mn=np.mean(icemass_month,axis=0)
		icemass_Scenario_stdev=np.std(icemass_month,axis=0)	

		# Combine the Hist and Scenario segments
		years=np.append(years_Hist,years_Scenario,axis=0)
		massbudget_mn=np.append(massbudget_Hist_mn,massbudget_Scenario_mn,axis=0)
		massbudget_stdev=np.append(massbudget_Hist_stdev,massbudget_Scenario_stdev,axis=0)
		icearea_mn=np.append(icearea_Hist_mn,icearea_Scenario_mn,axis=0)
		icearea_stdev=np.append(icearea_Hist_stdev,icearea_Scenario_stdev,axis=0)
		icemass_mn=np.append(icemass_Hist_mn,icemass_Scenario_mn,axis=0)
		icemass_stdev=np.append(icemass_Hist_stdev,icemass_Scenario_stdev,axis=0)

	# Otherwise just use the Hist section
	else:
		years = years_Hist
		massbudget_mn = massbudget_Hist_mn
		massbudget_stdev = massbudget_Hist_stdev
		icearea_mn = icearea_Hist_mn
		icearea_stdev = icearea_Hist_stdev
		icemass_mn = icemass_Hist_mn
		icemass_stdev = icemass_Hist_stdev


	return years,icearea_mn,icearea_stdev,icemass_mn,icemass_stdev,massbudget_mn,massbudget_stdev

#---------------------------------------------------------------------
#---------------------------------------------------------------------
#---------------------------------------------------------------------
def monthly_mn_snow_evolution(filepath,histfiles,nhistfiles,scenariofiles,nscenariofiles,nskip,month):

	# -----------------------------------------------------------------------------
	# Loop over the Hist files then loop over the scenario files
	# to make the monthly mn and st dev for a specified month,
	# for each budget component, plus the snow area and mass

	# Hist files --------------------------
	# Find the no of years
	years,months,snowarea,snowmass,massbudget = rb.read_snowfile(filepath+histfiles[0],nskip)
	nyears=len(years)
	massbudgets_month = np.empty([nhistfiles,nyears,7],dtype='float')
	snowarea_month = np.empty([nhistfiles,nyears],dtype='float')
	snowmass_month = np.empty([nhistfiles,nyears],dtype='float')


	# Make the annual mean budget for each ensemble member
	for n in range(0,nhistfiles):
		years_Hist,months,snowarea,snowmass,massbudget = rb.read_snowfile(filepath+histfiles[n],nskip)
		massbudgets_month[n,:,:] = massbudget[:,month-1,:]
		snowarea_month[n,:] = snowarea[:,month-1]
		snowmass_month[n,:] = snowmass[:,month-1]

	# Make the ensemble mean and st dev 
	massbudget_Hist_mn=np.mean(massbudgets_month,axis=0)
	massbudget_Hist_stdev=np.std(massbudgets_month,axis=0)
	snowarea_Hist_mn=np.mean(snowarea_month,axis=0)
	snowarea_Hist_stdev=np.std(snowarea_month,axis=0)	
	snowmass_Hist_mn=np.mean(snowmass_month,axis=0)
	snowmass_Hist_stdev=np.std(snowmass_month,axis=0)	


	# Include the Scenario files (if there are any....)----------------------------
	# Find the no of years
	if nscenariofiles > 0:
		years,months,snowarea,snowmass,massbudget = rb.read_snowfile(filepath+scenariofiles[0],nskip)
		nyears=len(years)
		massbudgets_month = np.empty([nscenariofiles,nyears,7],dtype='float')
		snowarea_month = np.empty([nscenariofiles,nyears],dtype='float')
		snowmass_month = np.empty([nscenariofiles,nyears],dtype='float')

		# Make the annual mean budget for each ensemble member
		for n in range(0,nscenariofiles):
			years_Scenario,months,snowarea,snowmass,massbudget = rb.read_snowfile(filepath+scenariofiles[n],nskip)
			massbudgets_month[n,:,:] = massbudget[:,month-1,:]
			snowarea_month[n,:] = snowarea[:,month-1]
			snowmass_month[n,:] = snowmass[:,month-1]


		# Make the ensemble mean and st dev
		massbudget_Scenario_mn=np.mean(massbudgets_month,axis=0)
		massbudget_Scenario_stdev=np.std(massbudgets_month,axis=0)
		snowarea_Scenario_mn=np.mean(snowarea_month,axis=0)
		snowarea_Scenario_stdev=np.std(snowarea_month,axis=0)	
		snowmass_Scenario_mn=np.mean(snowmass_month,axis=0)
		snowmass_Scenario_stdev=np.std(snowmass_month,axis=0)	

		# Combine the Hist and Scenario segments
		years=np.append(years_Hist,years_Scenario,axis=0)
		massbudget_mn=np.append(massbudget_Hist_mn,massbudget_Scenario_mn,axis=0)
		massbudget_stdev=np.append(massbudget_Hist_stdev,massbudget_Scenario_stdev,axis=0)
		snowarea_mn=np.append(snowarea_Hist_mn,snowarea_Scenario_mn,axis=0)
		snowarea_stdev=np.append(snowarea_Hist_stdev,snowarea_Scenario_stdev,axis=0)
		snowmass_mn=np.append(snowmass_Hist_mn,snowmass_Scenario_mn,axis=0)
		snowmass_stdev=np.append(snowmass_Hist_stdev,snowmass_Scenario_stdev,axis=0)

	# Otherwise just use the Hist section
	else:
		years = years_Hist
		massbudget_mn = massbudget_Hist_mn
		massbudget_stdev = massbudget_Hist_stdev
		snowarea_mn = snowarea_Hist_mn
		snowarea_stdev = snowarea_Hist_stdev
		snowmass_mn = snowmass_Hist_mn
		snowmass_stdev = snowmass_Hist_stdev


	return years,snowarea_mn,snowarea_stdev,snowmass_mn,snowmass_stdev,massbudget_mn,massbudget_stdev

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
def atmostemp_mn_evolution(filepath,histfiles,nhistfiles,scenariofiles,nscenariofiles,nskip):

	# -----------------------------------------------------------------------------
	# Loop over the Hist files then loop over the scenario files
	# to make the mn and st dev for the annual mean atmos temperature data


	# Hist files --------------------------
	# Find the no of years
	years_Hist,atmostemp = rb.read_atmostempfile(filepath+histfiles[0],nskip)
	nyears=len(years_Hist)
	atmostemp_annualmn = np.empty([nhistfiles,nyears],dtype='float')

	# Gather together data for all the Hist ensemble members
	for n in range(0,nhistfiles):
		years,atmostemp = rb.read_atmostempfile(filepath+histfiles[n],nskip)
		atmostemp_annualmn[n,:]=atmostemp

	# Make the Hist ensemble mean and st dev 
	atmostemp_Hist_mn=np.mean(atmostemp_annualmn,axis=0)
	atmostemp_Hist_stdev=np.std(atmostemp_annualmn,axis=0)
	
	# Include the Scenario files (if there are any....)----------------------------
	# Find the no of years
	if nscenariofiles > 0:
		years_Scenario,atmostemp = rb.read_atmostempfile(filepath+scenariofiles[0],nskip)
		nyears=len(years_Scenario)
		atmostemp_annualmn = np.empty([nscenariofiles,nyears],dtype='float')
		
		# Gather together data for all the Scenario ensemble members
		for n in range(0,nscenariofiles):
			years,atmostemp = rb.read_atmostempfile(filepath+scenariofiles[n],nskip)
			atmostemp_annualmn[n,:]=atmostemp

		# Make the Scenario ensemble mean and st dev 		
		atmostemp_Scenario_mn=np.mean(atmostemp_annualmn,axis=0)
		atmostemp_Scenario_stdev=np.std(atmostemp_annualmn,axis=0)


		# Combine the Hist and Scenario segments
		years=np.append(years_Hist,years_Scenario,axis=0)
		atmostemp_mn=np.append(atmostemp_Hist_mn,atmostemp_Scenario_mn,axis=0)
		atmostemp_stdev=np.append(atmostemp_Hist_stdev,atmostemp_Scenario_stdev,axis=0)

	# Otherwise just use the Hist section
	else:
		years = years_Hist
		atmostemp_mn=atmostemp_Hist_mn
		atmostemp_stdev=atmostemp_Hist_stdev

	return years,atmostemp_mn,atmostemp_stdev


#------------------------------------------------------------------------------------------------------------#------------------------------------------------------------------------------------------------------------
